#!/usr/bin/env python
_G=None
_F='port'
_E='addr'
_D='bind_addr'
_C='ip6'
_B='bind_port'
_A=False
import errno,socket
from socket import AF_INET,AF_INET6,IPPROTO_IPV6,IPPROTO_TCP,IPV6_V6ONLY,SO_REUSEADDR,SOCK_STREAM,SOL_SOCKET,TCP_NODELAY
from localstack.utils.server.proxy_server import start_tcp_proxy
from localstack.utils.threads import start_thread
class AttrDict(dict):
	def __init__(A,*B,**C):super(AttrDict,A).__init__(*(B),**C);A.__dict__=A
def tcp_listen(six,addr,port,blk):
	A=socket.socket(AF_INET6 if six else AF_INET,SOCK_STREAM)
	if six and hasattr(socket,'IPV6_V6ONLY'):A.setsockopt(IPPROTO_IPV6,IPV6_V6ONLY,0)
	A.setsockopt(SOL_SOCKET,SO_REUSEADDR,1);A.setsockopt(IPPROTO_TCP,TCP_NODELAY,1);A.bind((addr,port));A.listen(5);A.setblocking(blk);return A
def tcp_connect(six,addr,port,blk):A=socket.socket(AF_INET6 if six else AF_INET,SOCK_STREAM);A.setsockopt(IPPROTO_TCP,TCP_NODELAY,1);A.connect((addr,port));A.setblocking(blk);return A
def safe_close(x):
	try:x.close()
	except Exception:pass
class Server:
	def __init__(B,opt,q):A=opt;B.opt=A;B.sock=tcp_listen(A.ip6,A.bind_addr,A.bind_port,0);B.q=q
	def pre_wait(A,rr,r,w,e):r.append(A.sock)
	def post_wait(A,r,w,e):
		if A.sock in r:B,C=A.sock.accept();B.setblocking(0);A.q.append(Proxy(A.opt,B,C))
class Half:
	def __init__(A,opt,sock,addr,dir):A.opt=opt;A.sock=sock;A.addr=addr;A.dir=dir;A.name='peer'if A.dir else'client';A.queue=[];A.dest=_G;A.err=_G;A.ready=_A
	def pre_wait(A,rr,r,w,e):
		if A.ready:rr.append(A.sock)
		r.append(A.sock)
		if A.queue:w.append(A.sock)
	def post_wait(A,r,w,e):
		if not A.err and A.sock in w and A.queue:A.write_some()
		if not A.err and A.sock in r:A.ready=True;A.copy()
		return A.err
	def error(A,msg,e):A.err='Error on %s: %s: %s'%(A.name,msg,e);return A.err
	def write_some(A):
		try:B=A.sock.send(A.queue[0])
		except Exception as C:return A.error('send error',C)
		if B!=len(A.queue[0]):A.queue[0]=A.queue[0][B:]
		else:del A.queue[0]
	def copy(A):
		try:C=A.sock.recv(4096)
		except socket.error as B:
			if B.errno==errno.EWOULDBLOCK:A.ready=_A;return
			return A.error('recv socket error',B)
		except Exception as B:return A.error('recv error',B)
		if len(C)==0:return A.error('eof',0)
		A.dest.queue.append(C)
	def close(A):safe_close(A.sock)
class Proxy:
	def __init__(A,opt,sock,addr):B=opt;A.opt=B;A.cl=Half(B,sock,addr,'i');C=tcp_connect(B.ip6,B.addr,B.port,0);A.peer=Half(B,C,addr,'o');A.cl.dest=A.peer;A.peer.dest=A.cl;A.err=_G
	def pre_wait(A,rr,r,w,e):A.cl.pre_wait(rr,r,w,e);A.peer.pre_wait(rr,r,w,e)
	def post_wait(A,r,w,e):
		if not A.err:A.err=A.cl.post_wait(r,w,e)
		if not A.err:A.err=A.peer.post_wait(r,w,e)
		if A.err:A.cl.close();A.peer.close()
		return A.err
def server_loop(opt):A=opt;A[_C]=A.get(_C,_A);A[_D]=A.get(_D,'0.0.0.0');A[_E]=A.get(_E,'127.0.0.1');A[_B]=A.get(_B,0);B=f"{A[_D]}:{A[_B]}";C=f"{A[_E]}:{A[_F]}";D=start_thread(lambda *D,**A:start_tcp_proxy(B,C,handler=_G,**A));return D.join()
def main():A={_C:_A,_B:2223,_E:'192.168.43.191',_F:22};A={_C:_A,_B:48429,_E:'128.0.100.3',_F:22};A={_C:_A,_F:54237,_D:'128.0.100.2',_B:22};A={_C:_A,_F:54237,_D:'192.168.100.3',_B:22};server_loop(A)
if __name__=='__main__':main()


import sys
from datetime import datetime
from typing import IO, Iterable, List, Optional, Union

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ContentRangePattern = str
ContentType = str
ETag = str
ErrorMessage = str
ItemName = str
ListLimit = int
ListPathNaming = str
PaginationToken = str
PathNaming = str
RangePattern = str
SHA256Hash = str
StringPrimitive = str
statusCode = int


class ItemType(str):
    OBJECT = "OBJECT"
    FOLDER = "FOLDER"


class StorageClass(str):
    TEMPORAL = "TEMPORAL"


class UploadAvailability(str):
    STANDARD = "STANDARD"
    STREAMING = "STREAMING"


class ContainerNotFoundException(ServiceException):
    """The specified container was not found for the specified account."""

    code: str = "ContainerNotFoundException"
    sender_fault: bool = False
    status_code: int = 404


class InternalServerError(ServiceException):
    """The service is temporarily unavailable."""

    code: str = "InternalServerError"
    sender_fault: bool = False
    status_code: int = 400


class ObjectNotFoundException(ServiceException):
    """Could not perform an operation on an object that does not exist."""

    code: str = "ObjectNotFoundException"
    sender_fault: bool = False
    status_code: int = 404


class RequestedRangeNotSatisfiableException(ServiceException):
    """The requested content range is not valid."""

    code: str = "RequestedRangeNotSatisfiableException"
    sender_fault: bool = False
    status_code: int = 416


class DeleteObjectRequest(ServiceRequest):
    Path: PathNaming


class DeleteObjectResponse(TypedDict, total=False):
    pass


class DescribeObjectRequest(ServiceRequest):
    Path: PathNaming


TimeStamp = datetime
NonNegativeLong = int


class DescribeObjectResponse(TypedDict, total=False):
    ETag: Optional[ETag]
    ContentType: Optional[ContentType]
    ContentLength: Optional[NonNegativeLong]
    CacheControl: Optional[StringPrimitive]
    LastModified: Optional[TimeStamp]


class GetObjectRequest(ServiceRequest):
    Path: PathNaming
    Range: Optional[RangePattern]


PayloadBlob = bytes


class GetObjectResponse(TypedDict, total=False):
    Body: Optional[Union[PayloadBlob, IO[PayloadBlob], Iterable[PayloadBlob]]]
    CacheControl: Optional[StringPrimitive]
    ContentRange: Optional[ContentRangePattern]
    ContentLength: Optional[NonNegativeLong]
    ContentType: Optional[ContentType]
    ETag: Optional[ETag]
    LastModified: Optional[TimeStamp]
    StatusCode: statusCode


class Item(TypedDict, total=False):
    """A metadata entry for a folder or object."""

    Name: Optional[ItemName]
    Type: Optional[ItemType]
    ETag: Optional[ETag]
    LastModified: Optional[TimeStamp]
    ContentType: Optional[ContentType]
    ContentLength: Optional[NonNegativeLong]


ItemList = List[Item]


class ListItemsRequest(ServiceRequest):
    Path: Optional[ListPathNaming]
    MaxResults: Optional[ListLimit]
    NextToken: Optional[PaginationToken]


class ListItemsResponse(TypedDict, total=False):
    Items: Optional[ItemList]
    NextToken: Optional[PaginationToken]


class PutObjectRequest(ServiceRequest):
    Body: IO[PayloadBlob]
    Path: PathNaming
    ContentType: Optional[ContentType]
    CacheControl: Optional[StringPrimitive]
    StorageClass: Optional[StorageClass]
    UploadAvailability: Optional[UploadAvailability]


class PutObjectResponse(TypedDict, total=False):
    ContentSHA256: Optional[SHA256Hash]
    ETag: Optional[ETag]
    StorageClass: Optional[StorageClass]


class MediastoreDataApi:

    service = "mediastore-data"
    version = "2017-09-01"

    @handler("DeleteObject")
    def delete_object(self, context: RequestContext, path: PathNaming) -> DeleteObjectResponse:
        """Deletes an object at the specified path.

        :param path: The path (including the file name) where the object is stored in the
        container.
        :returns: DeleteObjectResponse
        :raises ContainerNotFoundException:
        :raises ObjectNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DescribeObject")
    def describe_object(self, context: RequestContext, path: PathNaming) -> DescribeObjectResponse:
        """Gets the headers for an object at the specified path.

        :param path: The path (including the file name) where the object is stored in the
        container.
        :returns: DescribeObjectResponse
        :raises ContainerNotFoundException:
        :raises ObjectNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("GetObject")
    def get_object(
        self, context: RequestContext, path: PathNaming, range: RangePattern = None
    ) -> GetObjectResponse:
        """Downloads the object at the specified path. If the object’s upload
        availability is set to ``streaming``, AWS Elemental MediaStore downloads
        the object even if it’s still uploading the object.

        :param path: The path (including the file name) where the object is stored in the
        container.
        :param range: The range bytes of an object to retrieve.
        :returns: GetObjectResponse
        :raises ContainerNotFoundException:
        :raises ObjectNotFoundException:
        :raises RequestedRangeNotSatisfiableException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("ListItems")
    def list_items(
        self,
        context: RequestContext,
        path: ListPathNaming = None,
        max_results: ListLimit = None,
        next_token: PaginationToken = None,
    ) -> ListItemsResponse:
        """Provides a list of metadata entries about folders and objects in the
        specified folder.

        :param path: The path in the container from which to retrieve items.
        :param max_results: The maximum number of results to return per API request.
        :param next_token: The token that identifies which batch of results that you want to see.
        :returns: ListItemsResponse
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutObject")
    def put_object(
        self,
        context: RequestContext,
        body: IO[PayloadBlob],
        path: PathNaming,
        content_type: ContentType = None,
        cache_control: StringPrimitive = None,
        storage_class: StorageClass = None,
        upload_availability: UploadAvailability = None,
    ) -> PutObjectResponse:
        """Uploads an object to the specified path. Object sizes are limited to 25
        MB for standard upload availability and 10 MB for streaming upload
        availability.

        :param body: The bytes to be stored.
        :param path: The path (including the file name) where the object is stored in the
        container.
        :param content_type: The content type of the object.
        :param cache_control: An optional ``CacheControl`` header that allows the caller to control
        the object's cache behavior.
        :param storage_class: Indicates the storage class of a ``Put`` request.
        :param upload_availability: Indicates the availability of an object while it is still uploading.
        :returns: PutObjectResponse
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError
